function points = CreateGeodesicPoints(numDivisions)
	% create a icosahedron first

	t1 = 2*pi/5;
	t2 = pi/10;
	t3 = -3*pi/10;
	t4 = pi/5;

	R = 1 ;
	side = 2*R*sin(t4);
	S = side;

	H = cos(t4)*R;
	Cx = R * cos(t2);
	Cy = R * sin(t2);

	H1 = sqrt(side*side - R*R);
	H2 = sqrt((H+R)*(H+R) - H*H);
	Z2 = (H2 - H1)/2;
	Z1 = Z2 + H1;

	points(1,:) = [0,  0, Z1];
	points(2,:) = [0,  R, Z2];
	points(3,:) = [Cx Cy Z2];
	points(4,:) = [S/2 -H Z2];
	points(5,:) = [-S/2 -H Z2];
	points(6,:) = [-Cx Cy Z2];
	points(7,:) = [0, -R -Z2];
	points(8,:) = [-Cx -Cy -Z2];
	points(9,:) = [-S/2 H -Z2];
	points(10,:) = [S/2  H -Z2];
	points(11,:) = [Cx -Cy -Z2];
	points(12,:) = [0  0 -Z1];

    face(1,:) = [1 2 3] ; 
	face(2,:) = [1 3 4] ;
	face(3,:) = [1 4 5] ;
	face(4,:) = [1 5 6] ;
	face(5,:) = [1 6 2] ;
	face(6,:) = [2 10 3] ;
	face(7,:) = [3 10 11] ;
	face(8,:) = [3 11 4] ;
	face(9,:) = [4 11 7] ;
	face(10,:) = [4 7 5] ;
	face(11,:) = [5 7 8] ;
	face(12,:) = [5 8 6] ;
	face(13,:) = [6 8 9] ;
	face(14,:) = [6 9 2] ;
	face(15,:) = [2 9 10] ;
	face(16,:) = [10 12 11] ;
	face(17,:) = [11 12 7] ;
	face(18,:) = [7 12 8] ;
	face(19,:) = [8 12 9] ;
	face(20,:) = [9 12 10] ;
	

	% now we have our icosahedron
	% let's extend it to a geodesic sphere
	% subdivide each face
	for subdiv=1:numDivisions
        numCurrentFaces = size(face,1) ; 
        numCurrentPoints = size(points,1) ; 
        faceNew = zeros(0,3) ; 
		for i=1:numCurrentFaces
			f = face(i,:);
			newV1 = 0.5*(points(f(1),:) + points(f(2),:));
			newV2 = 0.5*(points(f(2),:) + points(f(3),:));
			newV3 = 0.5*(points(f(3),:) + points(f(1),:));

			% create the 3 new faces
			id0 = f(1);
			id1 = f(2);
			id2 = f(3);

            idN1 = size(points,1) + 1; 
			for j=1:size(points,1)
				if sum(points(j,:) == newV1) == 3
					idN1 = j ;
					break;
                end
            end
            if idN1 == size(points,1) + 1 
                points(size(points,1)+1,:) = newV1 ; 
            end

            idN2 = size(points,1) + 1; 
			for j=1:size(points,1)
				if sum(points(j,:) == newV2) == 3
					idN2 = j ;
					break;
                end
            end
            if idN2 == size(points,1) + 1 
                points(size(points,1)+1,:) = newV2 ; 
            end

            idN3 = size(points,1) + 1; 
			for j=1:size(points,1)
				if sum(points(j,:) == newV3) == 3
					idN3 = j ;
					break;
                end
            end
            if idN3 == size(points,1) + 1 
                points(size(points,1)+1,:) = newV3 ; 
            end


            faceNew(size(faceNew,1)+1,:) = [id0 idN1 idN3];
            faceNew(size(faceNew,1)+1,:) = [idN1 id1 idN2];
            faceNew(size(faceNew,1)+1,:) = [idN2 id2 idN3];
            faceNew(size(faceNew,1)+1,:) = [idN1 idN2 idN3];
        end
        face = faceNew ; 
    end

	for i=1:size(points,1) 
        points(i,:) = points(i,:) * R / norm(points(i,:)) ; 
    end
