//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#include "RotationMatrix.h" 
#include <float.h>
#include <ostream> 


Point GridIntersection::Z_DIRECTION(0,0,1) ; 

GridIntersection::GridIntersection(void)
{
}

GridIntersection::~GridIntersection(void)
{
}
void GridIntersection::ComputeGridIntersectionPoints(std::vector<Point> &vectIntersectPoints, 
	const RotationMatrix &rotMatrix, float x, float y, int gridSide)
{
	// Box is a cube of side 2, centered at the the origin. 
	// Each voxel has size = 2 / (gridSide-1).
	const double GRID_SIDE = 2.0/(gridSide-1);
	Point pixelPt(x,y,0) ; 
	//Point zDirection = rotMatrix.InverseRotate(GridIntersection::Z_DIRECTION) ; 
	Point zDirection = rotMatrix.Rotate(GridIntersection::Z_DIRECTION) ; 
	// in the grid coordinates. 
	//Point pixelPtOrig = rotMatrix.InverseRotate(pixelPt) ; 
	Point pixelPtOrig = rotMatrix.Rotate(pixelPt) ; 

	// now start moving from the start Point in negative z Direction. 
	// after that we'll move in the positive z Direction. 
	bool increasingWithT[3] ; 
	bool isZero[3] ; 

	isZero[0] = isZero[1] = isZero[2] = false ; 
	if (zDirection.mfltX > 0)
		increasingWithT[0] = true ; 
	else
	{
		increasingWithT[0] = false ; 
		if (zDirection.mfltX == 0)
			isZero[0] = true ; 
	}
	if (zDirection.mfltY > 0)
		increasingWithT[1] = true ; 
	else
	{
		increasingWithT[1] = false ; 
		if (zDirection.mfltY == 0)
			isZero[1] = true ; 
	}
	if (zDirection.mfltZ > 0)
		increasingWithT[2] = true ; 
	else
	{
		increasingWithT[2] = false ; 
		if (zDirection.mfltZ == 0)
			isZero[2] = true ; 
	}

	float tX ,tY, tZ ; 
	tX = tY = tZ = -FLT_MAX ; 
	// start at the negative end of t that hits a boundary. 
	if (!isZero[0])
	{
		tX = std::min<float>((float)((-1.0f-pixelPtOrig.mfltX)/zDirection.mfltX), 
			(float)((1.0f-pixelPtOrig.mfltX)/zDirection.mfltX)) ; 
	}
	if (!isZero[1])
	{
		tY = std::min<float>((float)((-1.0f-pixelPtOrig.mfltY)/zDirection.mfltY), 
			(float)((1.0f-pixelPtOrig.mfltY)/zDirection.mfltY)) ;
	}
	if (!isZero[2])
	{
		tZ = std::min<float>((float)((-1.0f-pixelPtOrig.mfltZ)/zDirection.mfltZ),
			(float)((1.0f-pixelPtOrig.mfltZ)/zDirection.mfltZ)) ;
	}
	float tCurrent = std::max<float>(tX, std::max<float>(tY, tZ)) ; 

	// now tCurrent puts a point on the boundary.
	Point lastPoint = pixelPtOrig + tCurrent * zDirection ; 
	// stupid precision stuff. 
	if (tCurrent == tX)
	{
		if (increasingWithT[0])
			lastPoint.mfltX = -1.0f ; 
		else if (!isZero[0])
			lastPoint.mfltX = 1.0f ; 
	}
	if (tCurrent == tY)
	{
		if (increasingWithT[1])
			lastPoint.mfltY = -1.0f ; 
		else if (!isZero[1])
			lastPoint.mfltY = 1.0f ; 
	}
	if (tCurrent == tZ)
	{
		if (increasingWithT[2])
			lastPoint.mfltZ = -1.0f ; 
		else if (!isZero[2])
			lastPoint.mfltZ = 1.0f ; 
	}

	if (!lastPoint.OnGridBoundary())
		return ; 


	vectIntersectPoints.push_back(lastPoint) ; 

	float intPart = 0 ; 

	// the next indices we want to get to.
	int xIndex=0, yIndex=0, zIndex=0 ; 
	float nextX = 0 ; 

	if (increasingWithT[0])
	{
		xIndex = 1+ (int) ((lastPoint.mfltX + 1) *(gridSide-1) / 2) ; 
		nextX = -1 + (xIndex * 2.0f) / (gridSide-1) ; 
		// damned precision issues. Still need to check, in case xIndex was calculated wrong.
		if (nextX <= lastPoint.mfltX)
		{
			xIndex++ ; 
			nextX = -1 + (xIndex * 2.0f) / (gridSide-1) ; 
		}
	}
	else
	{
		xIndex = (int) ((lastPoint.mfltX + 1)*(gridSide-1) / 2) ; 
		nextX = -1 + (xIndex * 2.0f) / (gridSide-1) ; 
		if (nextX >= lastPoint.mfltX)
		{
			xIndex-- ; 
			nextX = -1 + (xIndex * 2.0f) / (gridSide-1) ; 
		}
	}

	float nextY = 0 ; 
	if (increasingWithT[1])
	{
		yIndex = 1+ (int) ((lastPoint.mfltY + 1) *(gridSide-1) / 2) ; 
		nextY = -1.0f + (yIndex * 2.0f) / (gridSide-1) ; 
		// damned precision issues. Still need to check, in case yIndex was calculated wrong.
		if (nextY <= lastPoint.mfltY)
		{
			yIndex++ ; 
			nextY = -1.0f + (yIndex * 2.0f) / (gridSide-1) ; 
		}
	}
	else
	{
		yIndex = (int) ((lastPoint.mfltY + 1)*(gridSide-1) / 2) ; 
		nextY = -1 + (yIndex * 2.0f) / (gridSide-1) ; 
		if (nextY >= lastPoint.mfltY)
		{
			yIndex-- ; 
			nextY = -1 + (yIndex * 2.0f) / (gridSide-1) ; 
		}
	}

	float nextZ = 0 ; 
	if (increasingWithT[2])
	{
		zIndex = 1+ (int) ((lastPoint.mfltZ + 1) *(gridSide-1) / 2) ; 
		nextZ = -1.0f + (zIndex * 2.0f) / (gridSide-1) ; 
		// damned precision issues. Still need to check, in case zIndex was calculated wrong.
		if (nextZ <= lastPoint.mfltZ)
		{
			zIndex++ ; 
			nextZ = -1.0f + (zIndex * 2.0f) / (gridSide-1) ; 
		}
	}
	else
	{
		zIndex = (int) ((lastPoint.mfltZ + 1)*(gridSide-1) / 2) ; 
		nextZ = -1.0f + (zIndex * 2.0f) / (gridSide-1) ; 
		if (nextZ >= lastPoint.mfltZ)
		{
			zIndex-- ; 
			nextZ = -1.0f + (zIndex * 2.0f) / (gridSide-1) ; 
		}
	}


	float tDeltaX, tDeltaY, tDeltaZ ; 
	float tMin = FLT_MAX ; 
	bool first = true ; 
	while(first || lastPoint.InsideGridBoundary())
	{
		first = false ; 
		tDeltaX = tDeltaY = tDeltaZ = FLT_MAX ; 
		if (!isZero[0])
			tDeltaX = (nextX - lastPoint.mfltX)/ zDirection.mfltX ; 
		if (!isZero[1])
			tDeltaY = (nextY - lastPoint.mfltY)/ zDirection.mfltY ; 
		if (!isZero[2])
			tDeltaZ = (nextZ - lastPoint.mfltZ)/ zDirection.mfltZ ; 
		tMin = std::min<float>(std::min<float>(tDeltaX, tDeltaY), tDeltaZ) ;

		lastPoint += tMin * zDirection ; 

		if (!lastPoint.OnGridBoundary() && ! lastPoint.InsideGridBoundary())
			break ; 
		vectIntersectPoints.push_back(lastPoint) ; 

		if (!isZero[0])
		{
			if (nextX == lastPoint.mfltX)
			{
				if (increasingWithT[0])
					xIndex++ ;
				else
					xIndex-- ; 
				nextX = -1.0f + (xIndex * 2.0f) / (gridSide-1) ; 
			}
		}
		if (!isZero[1])
		{
			if (nextY == lastPoint.mfltY)
			{
				if (increasingWithT[1])
					yIndex++ ;
				else
					yIndex-- ; 
				nextY = -1.0f + (yIndex * 2.0f) / (gridSide-1) ; 
			}
		}
		if (!isZero[2])
		{
			if (nextZ == lastPoint.mfltZ)
			{
				if (increasingWithT[2])
					zIndex++ ;
				else
					zIndex-- ; 
				nextZ = -1.0f + (zIndex * 2.0f) / (gridSide-1) ; 
			}
		}

	}
}

std::ostream& operator<<(std::ostream &out, const Point &p)
{
	out<<p.mfltX<<","<<p.mfltY<<","<<p.mfltZ ; 
	return out ; 
}

