//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#pragma once
#include <vector> 
#include <math.h> 
#include <ostream>


class Point
{
public:
	float mfltX ; 
	float mfltY ; 
	float mfltZ ; 
	Point(float x, float y, float z):mfltX(x), mfltY(y), mfltZ(z){} ; 
	Point(){} ; 
	inline float Norm2Sq() { return mfltX*mfltX + mfltY*mfltY + mfltZ*mfltZ ; }
	inline float Norm() { return sqrt(mfltX*mfltX + mfltY*mfltY + mfltZ*mfltZ) ; }
	inline bool OnGridBoundary()
	{
		return fabs(mfltX) <= 1 && (fabs(mfltY) <= 1) && (fabs(mfltZ) <= 1) && (fabs(mfltX) == 1 || fabs(mfltY) == 1 || fabs(mfltZ) == 1) ; 
	}
	inline bool InsideGridBoundary()
	{
		return (fabs(mfltX) < 1 && fabs(mfltY) <= 1 && fabs(mfltZ) <= 1) ||
				(fabs(mfltX) <= 1 && fabs(mfltY) < 1 && fabs(mfltZ) <= 1) ||
				(fabs(mfltX) <= 1 && fabs(mfltY) <= 1 && fabs(mfltZ) < 1)  ; 
	}

	inline void operator+= (const Point &a)
	{
		mfltX += a.mfltX ; mfltY += a.mfltY ; mfltZ += a.mfltZ ; 
	}
} ; 
std::ostream& operator<<(std::ostream &out, const Point &p) ; 
inline Point operator+ (const Point& a, const Point& b)
{
	return Point(a.mfltX + b.mfltX, a.mfltY + b.mfltY, a.mfltZ + b.mfltZ) ;
}
inline Point operator- (const Point& a, const Point& b)
{
	return Point(a.mfltX - b.mfltX, a.mfltY - b.mfltY, a.mfltZ - b.mfltZ) ;
}
inline Point operator* (float a, const Point& pt)
{
	return Point(a * pt.mfltX, a * pt.mfltY, a * pt.mfltZ) ;
}
inline Point operator* (const Point& pt, float a)
{
	return Point(a * pt.mfltX, a * pt.mfltY, a * pt.mfltZ) ;
}
inline Point operator* (double a, const Point& pt)
{
	return Point((float)(a * pt.mfltX), (float)(a * pt.mfltY), (float)(a * pt.mfltZ)) ;
}
inline Point operator* (const Point& pt, double a)
{
	return Point((float)(a * pt.mfltX), (float)(a * pt.mfltY), (float)(a * pt.mfltZ)) ;
}

class RotationMatrix ; 

class GridIntersection
{
public:
	static Point Z_DIRECTION ; 
	GridIntersection(void);
	~GridIntersection(void);
	static void ComputeGridIntersectionPoints(std::vector<Point> &vectIntersectPoints, 
		const RotationMatrix& rotMatrix, float x, float y, int gridSide) ; 
};
