//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#include "ProjectionData.h" 
#include "RotationMatrix.h" 
#include <iostream> 
#include <fstream> 
#include <time.h>
#include <sstream> 
#include <string> 
#include <float.h>

void CreateGeodesicGridOfRotations(int numGeodesicPts, float psiDeg, 
								   std::vector<RotationMatrix> &vectRotMatrices)
{
	RotationMatrix::CreateGeodesicRotations(numGeodesicPts, psiDeg, vectRotMatrices) ;
}

void PrintGeoDesicMatrices(const int numGeodesicPts, int psiDeg)
{
	std::stringstream ss;
	ss<<std::string("nGeo")<<numGeodesicPts<<"PsiDeg"<<psiDeg<<std::string("Matrices.csv") ; 
	std::string fileName ; 
	ss>>fileName ; 
	std::ofstream fout(fileName.c_str()) ; 
	std::vector<RotationMatrix> vectRotMatrices ; 
	CreateGeodesicGridOfRotations(numGeodesicPts, (float)psiDeg, vectRotMatrices) ;
	for (unsigned int i = 0 ; i < vectRotMatrices.size() ; i++)
		fout<<vectRotMatrices[i]<<std::endl ; 
	fout.close() ; 
}
void LoadRotationMatrices(const std::string &fileName, 
	std::vector<RotationMatrix> &vectRotMatrices)
{
	float rotMatrix[3][3] ; 
	char buffer[8] ; 
	std::ifstream fin(fileName.c_str()) ; 
	char sep ; 
	while (!fin.eof())
	{
		fin>>rotMatrix[0][0]>>sep>>rotMatrix[0][1]>>sep>>rotMatrix[0][2]>>sep ; 
		fin>>rotMatrix[1][0]>>sep>>rotMatrix[1][1]>>sep>>rotMatrix[1][2]>>sep ; 
		fin>>rotMatrix[2][0]>>sep>>rotMatrix[2][1]>>sep>>rotMatrix[2][2] ; 
		fin.getline(buffer, 32) ; 
		if (!fin.fail())
		{
			RotationMatrix mat(rotMatrix) ; 
			vectRotMatrices.push_back(mat) ; 
		}
	}
}

std::vector<std::string> &split(const std::string &s, 
		char delim, 
		std::vector<std::string> &elems) 
{    
	std::stringstream ss(s);    
	std::string item;    
	while(std::getline(ss, item, delim))
	{ 
		elems.push_back(item);    
	}    
	return elems;
}


