//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#pragma once
#include "TrilinearGridModelProbabilityComputer.h" 
#include "lbfgs.h"

class LBFGSFitTrilinearModel
{
protected:
    lbfgsfloatval_t *m_x;
	CryoData::ProjectionData *mobjData ; 
	TrilinearGridModelProbabilityComputer *mobjProbabilityModel ; 
	float mdblRegularizer ; 
	float mdblPriorLambda ; 
	bool mblnFixImage1 ; 
	bool mblnFixRotations ; 
	float mfltNoiseSigma ;
	int mintNumIterations ;
	bool mblnSaveProjections ; 

public:
	LBFGSFitTrilinearModel(void);
	virtual ~LBFGSFitTrilinearModel(void);
	 int FitModel(const std::string &mrcFile, 
		const std::string &analysisName, 
		const std::string &rotationFile,
		const std::string startModelMRCFile, 
		int numIterations, float regularizer,
		bool blnFixFirstImage, 
		int numGeodesicPts, float psiDeg, 
		float priorLambda,
		bool blnFixRotations, bool blnSaveProjections,
		const float startSDForAnnealing, 
		const float minSDForAnnealing,
		const float epsilon) ; 
protected:
    static lbfgsfloatval_t _evaluate(
        void *instance,
        const lbfgsfloatval_t *x,
        lbfgsfloatval_t *g,
        const int n,
        const lbfgsfloatval_t step
        )
    {
        return reinterpret_cast<LBFGSFitTrilinearModel*>(instance)->evaluate(x, g, n, step);
    }

    lbfgsfloatval_t evaluate(
        const lbfgsfloatval_t *x,
        lbfgsfloatval_t *g,
        const int n,
        const lbfgsfloatval_t step
        ) ;

    static int _progress(
        void *instance,
        const lbfgsfloatval_t *x,
        const lbfgsfloatval_t *g,
        const lbfgsfloatval_t fx,
        const lbfgsfloatval_t xnorm,
        const lbfgsfloatval_t gnorm,
        const lbfgsfloatval_t step,
        int n,
        int k,
        int ls
        )
    {
        return reinterpret_cast<LBFGSFitTrilinearModel*>(instance)->progress(x, g, fx, xnorm, gnorm, step, n, k, ls);
    }

    int progress(
        const lbfgsfloatval_t *x,
        const lbfgsfloatval_t *g,
        const lbfgsfloatval_t fx,
        const lbfgsfloatval_t xnorm,
        const lbfgsfloatval_t gnorm,
        const lbfgsfloatval_t step,
        int n,
        int k,
        int ls) ; 
};
