//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#pragma once
#include <string> 
#include <ostream> 

namespace CryoData
{
	class ProjectionData
	{
		bool mblnDensity ; 
		int mintNumProjections ; 
		int mintNumX ; 
		int mintNumY ;
		// 3 D array of data. 1st dimension is Z or projection #. Then its x,y dimensions.
		// It is indeed strange to have marrData[z][x][y] as the intensity of 
		// the (x,y,z)'th pixel. But it harominizes the way in which we store a set 
		// of projections with the way in which we store a density.
		float ***marrData ; 
		float mfltMinValue ; 
		float mfltMaxValue ; 
		void AllocateArrays() ; 
		void DeAllocateArrays() ; 
		inline int GetIndexInMRCArray(int xPix, int yPix) const { return yPix * mintNumX + xPix ; } ;
	public:
		ProjectionData():mintNumProjections(0), mintNumX(0), mintNumY(0), marrData(0), mblnDensity(false),
			mfltMinValue(-1.0f), mfltMaxValue(1.0f){}
		ProjectionData(const std::string &fName, bool isDensity=false);
		~ProjectionData(void);
		void PrintData(std::ostream &out, int projectionNum=-1) const ;
		const float*** GetDataPtr() const {return (const float ***)marrData ;} ; 
		float*** GetDataPtr() {return marrData ;} ; 
		int GetNumX() const { return mintNumX ; } ; 
		int GetNumProjections() const { return mintNumProjections ; } 
		void SetData(int numProjections, int imageSide, const float *** projections, 
			bool blnIsDensity=false) ; 
		void SetData(int numProjections, int imageSide, const double *** projections, 
			bool blnIsDensity=false);
		void WriteMRC(const std::string &outFile) ;
		static FILE* OpenMRCForWritingProjections(const std::string &outFile, 
			int numX, int numY, int numZ) ; 
		static void AddProjectionToOpenFile(FILE *fp, int numX, int numY, 
			const float **(&projection)) ; 
		static void AddProjectionToOpenFile(FILE *fp, 
			int numX, int numY, const double **(&projection)) ; 

		// rescales everything to 1. 
		void Rescale() ; 
	};
}
