#include "RBFGridModel.h"
#include "RotationMatrix.h"
#include "ProjectionData.h" 
#include "ran.h"
#include <iostream> 
#include <algorithm> 
#include <sstream>
#include <errno.h> 
#include <float.h>
#include <time.h>


const float LG_SQRT_TWO_PI = 0.5f * log(8.0f*(float)atan(1.0) ) ; 


RBFGridModel::RBFGridModel(
		const std::vector<RotationMatrix> &vectRotMatrices,
		unsigned char gridSide, 
		unsigned char imageSide): mbyteGridSide(gridSide), mbyteImageSide(imageSide),
			mvectRotations(vectRotMatrices)
{
	mblnUseRandomRotations = false ; 
	mfltRadius = 2.0f/(mbyteGridSide-1)/3 ; 
	mfltMaxDist = 3*mfltRadius ; 
}
void RBFGridModel::SetRotations(const std::vector<RotationMatrix> &vectRotMatrices)
{
	mvectRotations.clear() ; 
	mvectRotations.insert(mvectRotations.begin(), vectRotMatrices.begin(), vectRotMatrices.end()); 
}
RBFGridModel::RBFGridModel(char *fileName)
{
}

RBFGridModel::~RBFGridModel(void)
{
}


void RBFGridModel::GetGridContributions(
		const unsigned int rotMatrixIndex, 
		std::vector<TrilinearGridContribution> &vectGridContributions,
		const RotationMatrix *randRotMultiplier) 
{
	if (rotMatrixIndex > mvectRotations.size()-1)
		throw "Wanted rotation index greater than number of matrices provided" ; 

	RotationMatrix rotMatrix = mvectRotations[rotMatrixIndex] ; 
	if (randRotMultiplier != NULL)
	{
		rotMatrix = rotMatrix * (*randRotMultiplier); 
	}

	vectGridContributions.clear() ; 
	vectGridContributions.reserve(mbyteImageSide*mbyteImageSide*mbyteImageSide*8) ; 


	int numCopied = 0 ; 
	Point gridPoint ; 
	TrilinearGridContribution contrib ; 
	for (unsigned char xPix = 0 ; xPix < mbyteGridSide ; xPix++)
	{
		gridPoint.mfltX = -1.0f + (2.0f*xPix)/(mbyteGridSide-1) ; 
		for (int yPix = 0 ; yPix < mbyteGridSide ; yPix++)
		{
			gridPoint.mfltY = -1.0f + (2.0f*yPix)/(mbyteGridSide-1) ;
			for (int zPix = 0 ; zPix < mbyteGridSide ; zPix++)
			{
				contrib.mbyteXIndex = xPix ; contrib.mbyteYIndex = yPix ; contrib.mbyteZIndex = zPix ; 

				gridPoint.mfltZ = -1.0f + (2.0f*zPix)/(mbyteGridSide-1) ;
				Point newPoint = rotMatrix.Rotate(gridPoint) ; 
				// now go outwards from where we are and compute contributions. 
				int imgXIndex = (int) ((newPoint.mfltX + 1) *(mbyteImageSide-1) / 2 + .5) ; 
				int imgYIndex = (int) ((newPoint.mfltY + 1) *(mbyteImageSide-1) / 2 + .5) ; 
				
				// go backwards till xIndex or yIndex is too small. 
				int xStart = imgXIndex, yStart = imgYIndex ; 
				while (xStart > 0 && (-1.0f+(2.0f*xStart)/(mbyteImageSide-1) >= newPoint.mfltX-mfltMaxDist))
					xStart-- ; 
				if (-1.0f+(2.0f*xStart)/(mbyteImageSide-1) < newPoint.mfltX-mfltMaxDist)
					xStart++ ; 
				while (yStart > 0 && (-1.0f+(2.0f*yStart)/(mbyteImageSide-1) >= newPoint.mfltY-mfltMaxDist))
					yStart-- ; 
				if (-1.0f+(2.0f*yStart)/(mbyteImageSide-1) < newPoint.mfltY-mfltMaxDist)
					yStart++ ; 

				if (xStart < 0)
					xStart = 0 ; 
				if (yStart < 0)
					yStart = 0 ; 

				// now save contributions.
				imgXIndex = xStart ; 
				while (imgXIndex < mbyteImageSide)
				{
					float imageX = -1.0f+(2.0f*imgXIndex)/(mbyteImageSide-1) ; 
					if (imageX > newPoint.mfltX + mfltMaxDist)
						break ; 
					imgYIndex = yStart ; 
					while (imgYIndex < mbyteImageSide)
					{
						float imageY = -1.0f+(2.0f*imgYIndex)/(mbyteImageSide-1) ; 
						if (imageY > newPoint.mfltY + mfltMaxDist)
							break ; 
						// have x, y indices. add to contribution.
						contrib.mbyteImageXIndex = imgXIndex ; contrib.mbyteImageYIndex = imgYIndex ;
						float distanceSq = (imageX-newPoint.mfltX)*(imageX-newPoint.mfltX) + 
							(imageY-newPoint.mfltY)*(imageY-newPoint.mfltY) ; 
						contrib.mfltContribution = mfltRadius*exp(-0.5f*distanceSq/(mfltRadius*mfltRadius)) ; 
						vectGridContributions.push_back(contrib) ; 
						imgYIndex++ ; 
					}
					imgXIndex++ ; 
				}
			}
		}
	}
}
