//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#pragma once 
#include "GridIntersection.h"

class TrilinearGridContribution
{
public: 
	unsigned char mbyteXIndex ; 
	unsigned char mbyteYIndex ; 
	unsigned char mbyteZIndex ; 
	unsigned char mbyteImageXIndex ; 
	unsigned char mbyteImageYIndex ; 
	double mdblContribution ; 
	TrilinearGridContribution(unsigned char xIndex, unsigned char yIndex, 
		unsigned char zIndex, unsigned char imageXIndex, 
		unsigned char imageYIndex, double contribution): 
			mbyteXIndex(xIndex), 
			mbyteYIndex(yIndex), 
			mbyteZIndex(zIndex), 
			mbyteImageXIndex(imageXIndex),
			mbyteImageYIndex(imageYIndex),
			mdblContribution(contribution) 
	{
	};

	TrilinearGridContribution(): 
		mbyteXIndex(255), 
		mbyteYIndex(255), 
		mbyteZIndex(255), 
		mbyteImageXIndex(255),
		mbyteImageYIndex(255),
		mdblContribution(0.0)
	{
	} ;

	TrilinearGridContribution(const TrilinearGridContribution &a): 
		mbyteXIndex(a.mbyteXIndex), 
		mbyteYIndex(a.mbyteYIndex), 
		mbyteZIndex(a.mbyteZIndex), 
		mbyteImageXIndex(a.mbyteImageXIndex), 
		mbyteImageYIndex(a.mbyteImageYIndex), 
		mdblContribution(a.mdblContribution)
	{
	} ;
	TrilinearGridContribution& operator=(const TrilinearGridContribution &a)
	{
		if (this == &a)
			return *this ; 
		mbyteXIndex = a.mbyteXIndex ; 
		mbyteYIndex = a.mbyteYIndex ; 
		mbyteZIndex = a.mbyteZIndex ;
		mbyteImageXIndex = a.mbyteImageXIndex ;
		mbyteImageYIndex = a.mbyteImageYIndex ;
		mdblContribution = a.mdblContribution ; 
		return *this ; 
	}
	static inline double GetContributions(float px, float py, float pz, 
		const Point &p1, const Point &p2, float gridSide, double rayLength)
	{
		//time_t start = clock();
		double delX = p1.mfltX - px ; 
		double delY = p1.mfltY - py ; 
		double delZ = p1.mfltZ - pz ; 
		double dXEnds = p2.mfltX - p1.mfltX ; 
		double dYEnds = p2.mfltY - p1.mfltY ; 
		double dZEnds = p2.mfltZ - p1.mfltZ ; 

		if (p1.mfltX < px || p2.mfltX < px)
		{
			delX = -delX ; 
			dXEnds = -dXEnds ; 
		}
		if (p1.mfltY < py || p2.mfltY < py)
		{
			delY = -delY ; 
			dYEnds = -dYEnds ; 
		}
		if (p1.mfltZ < pz || p2.mfltZ < pz)
		{
			delZ = -delZ ; 
			dZEnds = -dZEnds ; 
		}

		double contrib = 1 - (2*(delX+delY+delZ)+dXEnds + dYEnds + dZEnds)/(2*gridSide)
			+  ((delX*(delY+delZ) + 0.5* (delX*(dYEnds+dZEnds) + (delY+delZ)*dXEnds) + dXEnds*(dYEnds+dZEnds)/3.0)
				+ (delZ*delY + 0.5* (delZ*dYEnds + delY*dZEnds) + dZEnds*dYEnds/3.0)
				)/(gridSide*gridSide)
			+ (delX*delY*delZ + 0.5*(dXEnds*delY*delZ + delX*dYEnds*delZ + delX*delY*dZEnds) 
									+ (dXEnds*dYEnds*delZ + dXEnds*delY*dZEnds + delX*dYEnds*dZEnds)/3.0
									+ 0.25 * dXEnds*dYEnds*dZEnds) / (gridSide*gridSide*gridSide) ; 

		return (contrib * rayLength) ; 
	}

	static void ComputePointContributions(const unsigned char imageXIndex, 
		const unsigned char imageYIndex, 
		const unsigned char gridSide, 
		const std::vector<Point> &vectPoints, 
		std::vector<TrilinearGridContribution> &vectGridContributions,
		RotationMatrix &rotMatrix) ; 
} ;
