//	This file is part of CryoBayes.
//  CryoBayes is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CryoBayes is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// CryoBayes: Copyright 2009,2010 Navdeep Jaitly
#pragma once
#include <vector>
#include "GridIntersection.h"
#include "TrilinearGridModel.h"
#include "time.h"
#include <string> 

class Complex ; 
class Convolver ; 
namespace CryoData
{
	class ProjectionData ; 
}


class TrilinearGridModelProbabilityComputer
{
	TrilinearGridModel mobjModel ; 
	double ***marrBetas ; 
	double ***marrCurrentProjections ; 
	double ***marrBetasTotalGradients ; 
	double ****marrImageBetaGradients ; 
	double ****marrImageBetaGradientsTemp ; 
	double **marrProjection ; 
	double **marrProjectionErrors ; 
	int mintCallNum ; 
	double mdblErrorSigma ; 
	double mdblPriorLambda ; 
	unsigned char mbyteGridSide ; 
	unsigned char mbyteImageSide ; 
	const float MAX_DIST_SQ_FROM_CENTER_IMAGE ; 
	const float MAX_DIST_SQ_FROM_CENTER_GRID ; 
	bool mblnFixImage1 ; 
	bool mblnFixRotations ;
	bool mblnSaveCurrentProjections ; 
	double mdblRegularizer ;
	double mdblCurrentLogPrior ; 
	short mshortNumImagesInData ; 
	std::vector<int> mvectProjectionsToUse ;
	std::vector<double> mvectImageErrorSquareForRotations ; 
	std::vector<double> mvectImageProbabilityProductPart ; 
	std::vector<double> mvectImageProbabilityExpPart ; 
	std::vector<double> mvectImageVarianceProductPart ; 
	std::string mstrAnalysisName ; 

	void DeAllocateMemory()  ;
	void AllocateMemory() ;
	void AllocateMemoryForCurrentProjections() ; 
	void DeAllocateMemoryForCurrentProjections() ; 
	void AllocateMemoryForProjectionImages() ;
	void DeAllocateMemoryForProjectionImages() ;


	void AllocateMemoryForImageGradients(int numImages) ;
	void DeAllocateMemoryForImageGradients() ; 

	void PrintCurrentScores() ; 
	void PrintCurrentModelProjections() ; 
	void PrintCurrentModel() ;

	void ClearMemoryForProjectionGradients(
		bool clearTemp, 
		bool clearGrads,
		bool clearTotalGradients) ;

	void ComputeCurrentProjection(const std::vector<TrilinearGridContribution> 
		&vectGridContributions) ; 

	double CalculateNextVariance() ;

	void ComputeRotationGradientsWithoutProbabilityProduct(
			const int rotationNum, 
			std::vector<TrilinearGridContribution> &vectGridContributions,
			std::vector<double> &vectImageErrorSquare, 
			const float ***IxyDataPtr) ; 
	void ComputeRotationGradientsWithProbabilityProduct(
		const int rotationNum, 
		const std::vector<double> &vectImageErrorSquare, 
		std::vector<double> &vectImageProbabilityProductPart,
		std::vector<double> &vectImageProbabilityExpPart,
		std::vector<double> &vectImageVarianceProductPart) ; 

	void ComputeRotationGradientsWithProbabilityProductFixedImages(
		const int rotationNum, 
		const std::vector<double> &vectImageErrorSquare, 
		std::vector<double> &vectImageProbabilityProductPart,
		std::vector<double> &vectImageProbabilityExpPart,
		std::vector<double> &vectImageVarianceProductPart) ; 

	double AddPriorRegularization(bool blnApply, double eps) ; 
	double CalculatePriorGradients(bool blnApply, double eps) ; 

	double TrainModelForOneIteration(
		const CryoData::ProjectionData &projData) ; 
public:
	TrilinearGridModelProbabilityComputer(
		const std::vector<RotationMatrix> &vectRotMatrices,
		unsigned char gridSide, 
		unsigned char imageSide);
	~TrilinearGridModelProbabilityComputer(void) ; 
	void GenerateRandomModel() ; 
	void GenerateSphericalModel() ; 
	void SetModelBetas(const unsigned char gridSide, 
			const float  ***data, 
			const unsigned char xColumn, 
			const unsigned char yColumn, 
			const unsigned char zColumn) ;
	void SetModelBetas(const unsigned char gridSide, 
			const double ***data, 
			const unsigned char xColumn, 
			const unsigned char yColumn, 
			const unsigned char zColumn) ;

	double ComputeLogLikelihood(
		const double *betas,
		double *gradients,
		CryoData::ProjectionData &projData, 
		double regularizer, 
		double priorLambda, 
		bool blnFixImage1,
		bool blnFixRotations, 
		bool blnSaveCurrentProjections,
		const float noiseSD) ; 

	void CopyBetas(double *(&betas)) ;
	void SetModelBetas(const double *(&betas)) ;
	void PrintModel(std::string fileName) ; 
	void PrintModelScores(std::string fileName) ; 
	void PrintBestProjections(std::string fileName) ; 
	void PrintBestRotations(std::string fileName) ; 
	double GetLastErrorSigma() { return mdblErrorSigma ; } ; 
};
